@@ -76,7 +76,7 @@ void DummyReviewsBackend::submitReview(AbstractResource* res, const QString& a,
     qDebug() << "dummy submit review" << res->name() << a << b << c;
 }
 
-bool DummyReviewsBackend::isResourceSupported(AbstractResource* res) const
+bool DummyReviewsBackend::isResourceSupported(AbstractResource* /*res*/) const
 {
     return true;
 }
